@@ -22,8 +22,9 @@ ff_inputs <- reactive({
 # Factor analysis
 ###############################
 output$ui_ff_vars <- renderUI({
-  isNum <- "numeric" == .get_class() | "integer" == .get_class()
-  vars <- varnames()[isNum]
+  vars <- varnames()
+  toSelect <- .get_class() %in% c("numeric", "integer", "date", "factor")
+  vars <- vars[toSelect]
   selectInput(
     inputId = "ff_vars", label = "Variables:", choices = vars,
     selected = state_multiple("ff_vars", vars, input$pf_vars),
@@ -36,48 +37,47 @@ output$ui_ff_store_name <- renderUI({
   textInput("ff_store_name", "Store factor scores:", "", placeholder = "Provide single variable name")
 })
 
-observe({
-  ## dep on most inputs
-  input$data_filter
-  input$show_filter
-  sapply(r_drop(names(ff_args)), function(x) input[[paste0("ff_", x)]])
-
-  ## notify user when the model needs to be updated
-  ## based on https://stackoverflow.com/questions/45478521/listen-to-reactive-invalidation-in-shiny
-  if (pressed(input$ff_run)) {
-    if (is.null(input$ff_vars)) { 
-      updateTabsetPanel(session, "tabs_full_factor", selected = "Summary")
-      updateActionButton(session, "ff_run", "Estimate model", icon = icon("play"))
-    } else if (isTRUE(attr(ff_inputs, "observable")$.invalidated)) {
-      updateActionButton(session, "ff_run", "Re-estimate model", icon = icon("refresh", class = "fa-spin"))
-    } else {
-      updateActionButton(session, "ff_run", "Estimate model", icon = icon("play"))
-    }
-  }
-})
+## add a spinning refresh icon if the tabel needs to be (re)calculated
+run_refresh(ff_args, "ff", init = "vars", tabs = "tabs_full_factor", label = "Estimate model", relabel = "Re-estimate model")
 
 output$ui_full_factor <- renderUI({
   req(input$dataset)
   tagList(
-    wellPanel(
-      actionButton("ff_run", "Estimate model", width = "100%", icon = icon("play"), class = "btn-success")
+    conditionalPanel(
+      condition = "input.tabs_full_factor == 'Plot'",
+      wellPanel(
+        checkboxGroupInput(
+          "ff_plots", NULL, c("Respondents" = "resp", "Attributes" = "attr"),
+          selected = state_group("ff_plots", "attr"),
+          inline = TRUE
+        )
+        # conditionalPanel(
+        #   condition = "input.tabs_full_factor == 'Plot'",
+        #   tags$table(
+        #     tags$td(numericInput("ff_scaling", "Respondent scale:", state_init("ff_scaling", 0.5), .5, 4, .1, width = "117px")),
+        #     tags$td(numericInput("ff_fontsz", "Font size:", state_init("ff_fontsz", 5), 1, 20, 1, width = "117px")),
+        #     width = "100%"
+        #   )
+        # )
+      )
     ),
     conditionalPanel(
       condition = "input.tabs_full_factor == 'Summary'",
+      wellPanel(
+        actionButton("ff_run", "Estimate model", width = "100%", icon = icon("play"), class = "btn-success")
+      ),
       wellPanel(
         uiOutput("ui_ff_vars"),
         selectInput(
           "ff_method", label = "Method:", choices = ff_method,
           selected = state_single("ff_method", ff_method, "PCA")
         ),
+        checkboxInput("ff_hcor", "Adjust for {factor} variables", value = state_init("ff_hcor", FALSE)),
         tags$table(
           tags$td(numericInput("ff_nr_fact", label = "Nr. of factors:", min = 1, value = state_init("ff_nr_fact", 1))),
           tags$td(numericInput("ff_cutoff", label = "Cutt-off:", min = 0, max = 1, value = state_init("ff_cutoff", 0), step = .05, width = "117px"))
         ),
-        conditionalPanel(
-          condition = "input.tabs_full_factor == 'Summary'",
-          checkboxInput("ff_fsort", "Sort factor loadings", value = state_init("ff_fsort", FALSE))
-        ),
+        checkboxInput("ff_fsort", "Sort factor loadings", value = state_init("ff_fsort", FALSE)),
         selectInput(
           "ff_rotation", label = "rotation:", ff_rotation,
           selected = state_single("ff_rotation", ff_rotation, "varimax")
@@ -101,7 +101,7 @@ output$ui_full_factor <- renderUI({
 })
 
 ff_plot <- reactive({
-  if (pressed(input$ff_run) && length(input$ff_vars) > 1 && 
+  if (pressed(input$ff_run) && length(input$ff_vars) > 1 &&
       isolate(input$ff_nr_fact) > 1) {
     plot_height <- plot_width <- 350
     nrFact <- min(isolate(input$ff_nr_fact), length(input$ff_vars))
@@ -113,7 +113,7 @@ ff_plot <- reactive({
 
     if (nrPlots > 1) {
       plot_width <- 700
-    } 
+    }
   } else {
     plot_height <- plot_width <- 700
   }
@@ -160,7 +160,7 @@ output$full_factor <- renderUI({
   if (not_pressed(input$ff_run)) {
     "** Press the Estimate button to generate factor analysis results **"
   } else if (not_available(input$ff_vars)) {
-    "This analysis requires multiple variables of type numeric or integer.\nIf these variables are not available please select another dataset.\n\n" %>% 
+    "This analysis requires multiple variables of type numeric or integer.\nIf these variables are not available please select another dataset.\n\n" %>%
      suggest_data("toothpaste")
   } else if (length(input$ff_vars) < 2) {
     "Please select two or more variables"
@@ -170,42 +170,51 @@ output$full_factor <- renderUI({
 })
 
 .full_factor <- eventReactive(input$ff_run, {
-  withProgress(
-    message = "Estimating factor solution", value = 1,
-    do.call(full_factor, ff_inputs())
-  )
+  withProgress(message = "Estimating factor solution", value = 1, {
+    ffi <- ff_inputs()
+    ffi$envir <- r_data
+    do.call(full_factor, ffi)
+  })
 })
 
 .summary_full_factor <- eventReactive({
-  input$ff_run
-  input$ff_cutoff
-  input$ff_fsort
+  c(input$ff_run, input$ff_cutoff, input$ff_fsort)
 }, {
   if (not_pressed(input$ff_run)) return("** Press the Estimate button to generate factor analysis results **")
   if (.ff_available() != "available") return(.ff_available())
   if (is_not(input$ff_nr_fact)) return("Number of factors should be >= 1")
+  validate(
+    need(
+      input$ff_cutoff >= 0 && input$ff_cutoff <= 1,
+      "Provide a correlation cutoff value in the range from 0 to 1"
+    )
+  )
   summary(.full_factor(), cutoff = input$ff_cutoff, fsort = input$ff_fsort)
 })
 
-.plot_full_factor <- reactive({
-  if (not_pressed(input$ff_run)) return("** Press the Estimate button to generate factor analysis results **")
-  isolate({
-    if (.ff_available() != "available") return(.ff_available())
-    if (is_not(input$ff_nr_fact) || input$ff_nr_fact < 2) return("Plot requires 2 or more factors.\nChange the number of factors in the Summary tab and re-estimate")
+.plot_full_factor <- eventReactive({
+  c(input$ff_run, !is.null(input$ff_plots))
+}, {
+  if (not_pressed(input$ff_run)) {
+    "** Press the Estimate button to generate factor analysis results **"
+  } else if (.ff_available() != "available") {
+    .ff_available()
+  } else if (is_not(input$ff_nr_fact) || input$ff_nr_fact < 2) {
+    "Plot requires 2 or more factors.\nChange the number of factors in the Summary tab and re-estimate"
+  } else {
     withProgress(message = "Generating factor plots", value = 1, {
-      plot(.full_factor(), shiny = TRUE)
-    }) 
-  })
+      plot(.full_factor(), plots = input$ff_plots, shiny = TRUE)
+    })
+  }
 })
 
 observeEvent(input$full_factor_report, {
   outputs <- c("summary", "plot")
   inp_out <- list(list(cutoff = input$ff_cutoff, fsort = input$ff_fsort, dec = 2), list(custom = FALSE))
   if (!is_empty(input$ff_store_name)) {
-    # xcmd <- paste0("store(result, name = \"", input$ff_store_name, "\")\n")
-    xcmd <- paste0(input$dataset, " <- store(", 
-      input$dataset, ", result, name = \"", input$ff_store_name, "\")"
-    )
+    fixed <- fix_names(input$ff_store_name)
+    updateTextInput(session, "ff_store_name", value = fixed)
+    xcmd <- glue('{input$dataset} <- store({input$dataset}, result, name = "{fixed}")')
   } else {
     xcmd <- ""
   }
@@ -223,23 +232,16 @@ observeEvent(input$full_factor_report, {
   )
 })
 
-## store factor scores
-# observeEvent(input$ff_store, {
-#   if (pressed(input$ff_run)) {
-#     .full_factor() %>% 
-#       {if (!is.character(.)) store(., name = input$ff_store_name)}
-#   }
-# })
-
 ## store factor scores
 observeEvent(input$ff_store, {
   req(input$ff_store_name, input$ff_run)
+  fixed <- fix_names(input$ff_store_name)
+  updateTextInput(session, "ff_store_name", value = fixed)
   robj <- .full_factor()
   if (!is.character(robj)) {
     withProgress(
       message = "Storing factor scores", value = 1,
-      r_data[[input$dataset]] <- store(r_data[[input$dataset]], robj, name = input$ff_store_name)
-      # store(robj, name = input$ff_store_name)
+      r_data[[input$dataset]] <- store(r_data[[input$dataset]], robj, name = fixed)
     )
   }
 })
@@ -256,16 +258,16 @@ dl_ff_loadings <- function(path) {
 }
 
 download_handler(
-  id = "dl_ff_loadings", 
-  fun = dl_ff_loadings, 
+  id = "dl_ff_loadings",
+  fun = dl_ff_loadings,
   fn = function() paste0(input$dataset, "_loadings"),
   type = "csv",
   caption = "Save factor loadings"
 )
- 
+
 download_handler(
-  id = "dlp_full_factor", 
-  fun = download_handler_plot, 
+  id = "dlp_full_factor",
+  fun = download_handler_plot,
   fn = function() paste0(input$dataset, "_factor"),
   type = "png",
   caption = "Save factor plots",
